//    IPS Java Utils
// 	  (c) Copyright 2012
// 	  Institute of Phonetics and Speech Processing,
//    Ludwig-Maximilians-University, Munich, Germany
//
//
//    This file is part of IPS Java Utils
//
//
//    IPS Java Utils is free software: you can redistribute it and/or modify
//    it under the terms of the GNU Lesser General Public License as published by
//    the Free Software Foundation, version 3 of the License.
//
//    IPS Java Utils is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public License
//    along with IPS Java Utils.  If not, see <http://www.gnu.org/licenses/>.

package ipsk.util.apps.ui;

import java.awt.Desktop;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridBagLayoutInfo;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import ipsk.swing.text.EditorKitMenu;
import ipsk.util.apps.descriptor.ApplicationVersionDescriptor;
import ipsk.util.apps.descriptor.InstallationPackage;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * @author klausj
 *
 */
public class ApplicationVersionDescriptorUI extends JPanel implements ActionListener {

    
    private ApplicationVersionDescriptor  applicationVersionDescriptor;
    private JButton downloadButton;
    private Desktop desktop;
    private URI downloadURI;
    public ApplicationVersionDescriptorUI(ApplicationVersionDescriptor avd){
        super(new GridBagLayout());
        GridBagConstraints gc=new GridBagConstraints();
        Insets insets=new Insets(1,1,1,1);
        gc.insets=insets;
        gc.gridx=0;
        gc.gridy=0;
        gc.fill=GridBagConstraints.HORIZONTAL;
        gc.weightx=2;
        this.applicationVersionDescriptor=avd;
        JLabel versionLabel=new JLabel(applicationVersionDescriptor.getVersion().toString());
        add(versionLabel,gc);
        InstallationPackage ip=applicationVersionDescriptor.getPlatformInstallationPackage();
        if(ip!=null){
        URL downloadURL=ip.getDownloadURL();
        if(downloadURL!=null){
            try {
                
                JTextField downloadUrlField=new JTextField(downloadURL.toString());
                downloadUrlField.setEditable(false);
                EditorKitMenu editorKitMenu=new EditorKitMenu(downloadUrlField,false);
                gc.gridx++;
                add(downloadUrlField,gc);
                downloadURI=downloadURL.toURI();
               
                if(Desktop.isDesktopSupported()){
                    desktop=Desktop.getDesktop();
                    if(desktop!=null){
                        downloadButton = new JButton("Download");
                        downloadButton.addActionListener(this);
                        gc.gridx++;
                        add(downloadButton,gc);
                    }
                }
            } catch (URISyntaxException e) {
                // OK simply ignore (no download button)
            }
        }
        }
    }
    
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        // TODO Auto-generated method stub

    }


    /* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        Object src=e.getSource();
        if(src==downloadButton && downloadURI!=null){
            
                try {
                    desktop.browse(downloadURI);
                } catch (IOException e1) {
                    JOptionPane.showMessageDialog(this, "Could not open "+downloadURI+": "+e1.getLocalizedMessage(), "Open download URL error", JOptionPane.ERROR_MESSAGE);
                }
            
        }
        
    }

}
